<?php

class KF_Api{    
    
    private $last_error;
    
    public function __construct()
    {
        
    }   
    public function get_last_error()
    {
        return $this->last_error;
    }
    
    private function call_api($name, $headers, $xml)
    {
        // standard headers
        $std_headers = array(
            'Content-Type'=> 'text/xml; charset=utf-8',
            'Accept' => 'text/xml',
            'Host' => 'securedwebapp.com'
        );
        // merge headers
        $headers = array_merge($std_headers, $headers);
        
            $params = array( 
                'body' => $xml,
                'method' => 'POST',
                'headers' => $headers,
                'sslverify' => false
            );   
            
            $result = wp_remote_post(DS_KASHFLOW_SOAP_URL, $params);
            if($result['response']['message']== "OK")
            {
                $soap_body = preg_replace("/(<\/?)(\w+):([^>]*>)/", "$1$2$3", $result['body']);
                $sxml = new SimpleXMLElement($soap_body, LIBXML_NOWARNING);
                $status = (string)$sxml->soapBody->{$name . "Response"}->Status[0];
                if($status == "NO")
                {
                    $this->last_error = (string)$sxml->soapBody->{$name . "Response"}->StatusDetail[0];
                    return false;
                }
                return (array)$sxml->soapBody->{$name . "Response"}->{$name . "Result"};
            }
            else
            {
                $this->last_error = __("Failed to connect to KashFlow server.", "ds-kashflow");
                return false;
            }
                
            
            //$sxml = new SimpleXMLElement($response['body']);
            return $response;
    }   
    private function call_soap_function($name, $xml = '')
    {
        $username = get_option('ds_kashflow_username');
        $password = get_option('ds_kashflow_api_password');
        
        
        $soap_xml  = '<?xml version="1.0" encoding="utf-8"?>';
        $soap_xml .= '<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">';
        $soap_xml .= '<soap12:Body>';
        
        $soap_xml .= '<' . $name . ' xmlns="KashFlow">';
        $soap_xml .= '<UserName>' . $username . '</UserName>';
        $soap_xml .= '<Password>' . $password . '</Password>';
        $soap_xml .= $xml;
        $soap_xml .= '</' . $name . '></soap12:Body></soap12:Envelope>';
        
        $headers = array(
            'Content-Length' => strlen($soap_xml),
            'SOAPAction' => '"KashFlow/' . $name . '"'
        );
        
        return $this->call_api($name, $headers, $soap_xml);
    }
    
    // API Calls
    
    public function get_company_details()
    {        
        return $this->call_soap_function("GetCompanyDetails");
    }
    
    public function getCustomerByEmail($email)
    {
        $xml = '<CustomerEmail>' . $email . '</CustomerEmail>';
        $res = $this->call_soap_function('GetCustomerByEmail', $xml);
        if($res)
        {
            $customer = new KF_Customer();
            $customer->populate($res);
            return $customer;
        }
        return false;        
    }
    
    public function update_customer($customer)
    {
        if($res = $this->getCustomerByEmail($customer->Email))
        {
            $id = $res->CustomerID;
            $customer->data = array_merge($res->data, $customer->data );
            //update existing customer
            $res = $this->call_soap_function('UpdateCustomer', $customer);
            $res = $id;
        }
        else
        {
            //create new customer
            $res = $this->call_soap_function('InsertCustomer', $customer);
            if(is_array($res)) $res = $res[0];
            
        }
        return $res;
    }
    public function insert_invoice($invoice, $lines = array())
    {
        $xml = '<Inv>';
        $xml .= (string)$invoice;
        
        if(count($lines)>0)
        {
            $xml .= '<Lines>';
            foreach($lines as $line)
            {
                $xml .= '<anyType xsi:type="InvoiceLine">';
                $xml .= (string)$line;
                $xml .= '</anyType>';
            }
            $xml .= '</Lines>';
        }
        $xml .= '</Inv>';
        $res = $this->call_soap_function('InsertInvoice', $xml);
        if(is_array($res)) $res = $res[0];
        return $res;
    }
    public function insert_invoice_line($invoice_id, $invoice_line)
    {
        $xml = '<InvoiceID>' . $invoice_id . '</InvoiceID>';
        $xml .= (string)$invoice_line;
        $res = $this->call_soap_function('InsertInvoiceLine', $xml);
        if(is_array($res)) $res = $res[0];
        return $res;
    }
    /**
    * Take Payment for an invoice
    * 
    * @param KFInvoicePayment $invoice_payment
    */
    public function insert_invoice_payment($invoice_payment)
    {
        $res = $this->call_soap_function('InsertInvoicePayment', $invoice_payment);        
        if(is_array($res)) $res = $res[0];
        return $res;        
    }
    public function get_inv_pay_methods()
    {
        $res = $this->call_soap_function('GetInvPayMethods');
        return $res;
    }
    
    /**
    * get the current nominal codes from KashFlow
    * 
    */
    public function get_nominal_codes()
    {
        return $this->call_soap_function('GetNominalCodes');
    }
    
    /**
    * get the current sales types from KashFlow
    * 
    */
    public function get_sales_types()
    {
        $sales_types = array();
        $types =  $this->call_soap_function('GetProducts');
        if(is_array($types))
        {
            if(is_array($types['Product']))
            {
                foreach($types['Product'] as $product)
                {
                    $sales_types[] = array(
                                        "ID" => (int)$product->ProductID, 
                                        'Name' => (string)$product->ProductName, 
                                        'Code' => (string)$product->ProductCode 
                                        );
                }    
            }
        }
        return $sales_types;
    }

    public function get_sub_products($nominal_id)
    {
        $xml = '<NominalID>' . $invoice_id . '</NominalID>';
        $res = $this->call_soap_function('GetSubProducts', $xml);
        return $res;
    }
    
    
    
}

?>
