<?php
/*
Plugin Name: Kashflow for Woocommerce
Plugin URI: http://devicesoftware.com/kashflow-for-woocommerce/
Description: Kashflow for woocommerce
Version: 0.0.2
Author: DeviceSoftware
Author URI: http://devicesoftware.com/kashflow-for-woocommerce/

Text Domain: ds-kashflow-for-woocommerce
Domain Path: /languages/ 

* 
*/

/*  Copyright 2012  Devicesoftware  (email : support@devicesoftware.com) 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA    

*
* 0.0.2 - 2013-05-27
*   Fix: Username & password API error resolved
*
* 0.0.1 - 2013-05-16
*   Initial release.
*/

// definitions
define("DS_KASHFLOW_SOAP_URL", "https://securedwebapp.com/api/service.asmx");
define("DS_KASHFLOW_ACCOUNTS", "accounts");
define("DS_KASHFLOW_PLUGINPATH", "/" . plugin_basename( dirname(__FILE__) ));

// check for abspath
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Ds_Kashflow' ) ) {
    
class Ds_Kashflow {   
    
    //current version
    protected $version = '0.0.2';    
    
    protected $plugin_path;        
    protected $plugin_url;    
    protected $template_url;
        
    //kashflow properties
    protected $last_error;
    protected $api;
    
    /**
    * plugin url
    * 
    */
    public function plugin_url()
    {
        return $this->plugin_url;
    }
    /**
    * plugin path
    * 
    */
    public function plugin_path()
    {
        return $this->plugin_path;
    }

    

    /**
    * constructor
    * 
    */
    public function __construct() {
        
        // Define version constant
        define( 'DS_KASHFLOW_VERSION', $this->version );
        
        // plugin path
        $this->plugin_path = dirname(__FILE__);
        
        // plugin url
        $this->plugin_url = plugins_url( basename( plugin_dir_path(__FILE__) ), basename( __FILE__ ) );
        
        // include files
        $this->includes();
        
        // create instance of the kashflow api
        $this->api = new KF_Api();
        
        if ( is_admin() && ! defined('DOING_AJAX') ) 
        {
            // Installation            
            register_activation_hook(__FILE__, array($this, 'install'));                        
            if ( get_option('ds_kashflow_db_version') !== $this->version )
            {
                add_action( 'init', array($this, 'install'), 1 );
            }          
            
            // modify settings tabs      
            
            add_action( 'woocommerce_settings_start', array($this, 'account_settings'));
            add_filter( 'woocommerce_settings_tabs_array', array($this, 'add_settings_tab'));
            add_action( 'woocommerce_settings_tabs_accounts', array($this, 'admin_settings'));            
            add_action( 'woocommerce_update_options_accounts', array($this, 'save_admin_settings'));
            add_action( 'woocommerce_admin_field_dstestapi', array($this, 'admin_field_testapi'));
            add_action( 'woocommerce_admin_field_label', array($this, 'admin_field_label'));     
        }
        
        add_action('woocommerce_checkout_order_processed', array($this, 'checkout_order_processed'), 10, 2);
        add_action('woocommerce_payment_complete', array($this, 'payment_complete'));

                
        add_action('wp_ajax_test_api', array($this, 'test_api_callback'));                        
                       
    } // end constructor
        
    private function includes()
    {
        include_once( 'classes/class-ds-api.php' );
        
        include_once( 'classes/class-ds-base.php' );
        include_once( 'classes/class-ds-customer.php' );
        include_once( 'classes/class-ds-invoice.php' );
        include_once( 'classes/class-ds-invoice-line.php' );
        include_once( 'classes/class-ds-invoice-payment.php' );
    }
    
    /**
    * install
    * 
    * currently not used in this version
    * 
    */
    public function install()
    {
        
    }

    /**
    * checkout_order_processed
    * 
    * this action runs once the order has been processed
    * 
    * @param int $order_id
    * @param array $posted
    */
    public function checkout_order_processed($order_id, $posted)
    {
        $order = new WC_Order($order_id);
        
        // update/add customer details
        
        // create a KashFlow customer class instance
        $customer = new KF_Customer();
        if(!empty($order->billing_company))
        {
            $customer->Name =  $order->billing_company;
        }
        else
        {
            $customer->Name =  $order->billing_first_name . ' ' .  $order->billing_last_name;
        }        
        $customer->Contact = $order->billing_first_name . ' ' .  $order->billing_last_name;
        $customer->ContactFirstName = $order->billing_first_name;
        $customer->ContactLastName = $order->billing_last_name;        
        $customer->Telephone = $order->billing_phone;
        $customer->Email = $order->billing_email;
        $customer->Address1 = $order->billing_address_1;
        $customer->Address2 = $order->billing_address_2;
        $customer->Address3 = $order->billing_city . ' ' . $order->billing_state;
        $customer->Address4 = $order->billing_country;
        $customer->Postcode = $order->billing_postcode;
        $customer->CustHasDeliveryAddress = 1;
        $customer->DeliveryAddress1 = $order->shipping_address_1;
        $customer->DeliveryAddress2 = $order->shipping_address_2;
        $customer->DeliveryAddress3 = $order->shipping_city . ' ' . $order->shipping_state;
        $customer->DeliveryAddress4 = $order->shipping_country;
        $customer->DeliveryPostcode = $order->shipping_postcode;
        
        // update KashFlow with customer information from WooCommerce Order
        $customer_id = $this->api->update_customer($customer);
        if($customer_id)
        {
            // get order totals
            $totals = $order->get_order_item_totals();
            $items = $order->get_items();
            $invoice = new KF_Invoice(!count($items));
            $invoice->InvoiceNumber = $order_id;
            $invoice->Customer = $customer->Name;
            $invoice->CustomerID =  $customer_id;
            $order_tax = $order->order_tax + $order->order_shipping_tax;
            $order_net = $order->order_total - $order_tax;
            $invoice->NetAmount = $order_net;
            $invoice->VATAmount = $order_tax;
            $total_amount = $order_net + $order_tax;
            $invoice->AmountPaid = $total_amount;
                        
            // add shipping item // TODO 4 -o swicks : always displays as order_shipping returns "0.00" if empty
            if($order->order_shipping)
            {
                $item = array();
                $item['charge_type'] = get_option('ds_kashflow_shipping_type');
                $item['name'] = __('Shipping cost', 'ds-kashflow');
                $item['qty'] = 1;
                $item['line_subtotal'] = $order->order_shipping;
                $item['line_tax'] = $order->order_shipping_tax;
                $items[] = $item;
            }

            // get sale of goods productID
            $default_sale_of_goods = get_option('ds_kashflow_sale_goods_type');
            
            $lines = array();
            $cnt = 0;
            foreach($items as $item)
            {
                $line = new KF_Invoice_Line(false);
                $line->Description = $item['name'];
                $qty = $item['qty'];
                $line->Quantity = $qty; 
                // HACK 0 -o swicks :Match the pennies / totals              
                $line_sub_total = $item['line_subtotal'];
                $unit_rate = round($line_sub_total / $qty, 2);
                $line_rate = $unit_rate * $qty;
                $line_total = $line_sub_total + $item['line_tax'];
                $line->Rate = $unit_rate;
                $line->VatAmount = $line_total - $line_rate;
                $line->ChargeType = isset($item['charge_type']) ? (int)$item['charge_type'] : $default_sale_of_goods;
                $line->LineID = $cnt++;
                // TODO 4 -o swicks : pull tax % from woocommerce
                $vat_rate = $order_tax ? 20 : 0;
                $line->VatRate = $vat_rate;
                
                // TODO 4 -o swicks : look into product ID
                //$line->ProductID = ????
                
                $lines[] = $line;
            }                        
            $invoice_id = $this->api->insert_invoice($invoice, $lines);
        }        
    }
    /**
    * payment complete
    * 
    * once payment has been taken update KashFlow
    * 
    * @param int $order_id
    */
    public function payment_complete($order_id)
    {
        $order = new WC_Order($order_id);
        $total_amount = $order->get_total();
        $invoice_payment = new KF_Invoice_Payment();
        $invoice_payment->PayInvoice = $order_id;
        $invoice_payment->PayAmount = $total_amount;        
        if($pay_method = get_option('ds_kashflow_gw_' . $order->payment_method))
            $invoice_payment->PayMethod = $pay_method;
        
        
        $invoice_response = $this->api->insert_invoice_payment($invoice_payment);
    }
    
    
    /**
    * splits an array and inserts a key/value
    * 
    * @param array $array
    * @param string $key
    * @param array $insert
    */
    private function array_insert($array, $key, $insert)
    {
        $res = array();
        foreach($array as $k => $v)
        {
            $res[$k] = $v;
            if($k == $key)
            {
                $res = array_merge_recursive($res, $insert);
            }
        }
        return $res;
    }
    /**
    * add settings tab to WooCommerce Settings
    * 
    * @param array $tabs
    */
    public function add_settings_tab($tabs)
    {        
        return $this->array_insert($tabs, 'tax', array(DS_KASHFLOW_ACCOUNTS => "Accounts"));        
    }
    
    /**
    * account_settings
    * 
    * array of options for the accounts tab
    * 
    */
    public function account_settings()
    {
        global $woocommerce_settings, $woocommerce;
        
        //sales types
        $sales_types_list = array();
        $sales_types = $this->api->get_sales_types();
        foreach($sales_types as $sales_type)
        {
            $sales_types_list[$sales_type['ID']] = $sales_type['Name'] . ' [' . $sales_type['Code'] . ']';
        }
        $gateway_list = array();
        $gateways = $woocommerce->payment_gateways();
        foreach($gateways->payment_gateways as $gateway)
        {
            if($gateway->settings['enabled'] == 'yes')
            {
                $gateway_list[$gateway->id] = $gateway->title;
            }            
        }
        $inv_pay_list = array();
        $inv_pay_methods = $this->api->get_inv_pay_methods();
        if(is_array($inv_pay_methods))
        {
            foreach($inv_pay_methods['PaymentMethod'] as $inv_pay_method)
            {
                $inv_pay_list[(string)$inv_pay_method->MethodID] = (string)$inv_pay_method->MethodName;
            }            
        }        
     
        $woocommerce_settings[DS_KASHFLOW_ACCOUNTS] = array(
            array( 'title' => __( 'Account Settings', 'ds-kashflow' ), 'type' => 'title', 'desc' => '', 'id' => 'ds_kashflow_settings' ), //section start
            array(
                'title' => __( 'UserName', 'ds-kashflow' ),
                'type'         => 'text',
                'desc'         => '',
                'id'         => 'ds_kashflow_username',                
                'css'         => 'min-width:300px;'
            ),
            array(
                'title' => __( 'API Password', 'ds-kashflow' ),
                'type'         => 'password',
                'desc'         => '',
                'id'         => 'ds_kashflow_api_password',                
                'css'         => 'min-width:300px;'
            ),
            array( 'type' => 'sectionend', 'id' => 'ds_kashflow_settings' ), //section end
            array( 'title' => __( 'Nominal Codes', 'ds-kashflow' ), 'type' => 'title', 'desc' => '', 'id' => 'ds_kashflow_nominal' ), //section start
            array(
                'title' => __('Sale of Goods', 'ds-kashflow'),
                'type' => 'select',
                'options' => $sales_types_list,
                'desc' => __( 'Select \'Sales Type\' for the products you are selling.', 'ds-kashflow' ),
                'id'         => 'ds_kashflow_sale_goods_type'
            ),
            array(
                'title' => __('Shipping Type', 'ds-kashflow'),
                'type' => 'select',
                'options' => $sales_types_list,
                'desc' => __( 'Select \'Sales Type\' for Shipping.', 'ds-kashflow' ),
                'id'         => 'ds_kashflow_shipping_type'
            ),
            array( 'type' => 'sectionend', 'id' => 'ds_kashflow_nominal' ) //section end
            ); 
                        
            $gws = array();
            $gws[] = array( 'title' => __( 'Active Payment Gateways', 'ds-kashflow' ), 'type' => 'title', 'desc' => '', 'id' => 'ds_wcgateways_settings' ); //section start
            foreach($gateway_list as $k => $v)
            {
                $gws[] = array(
                        'title' => $v,
                        'type' => 'select',
                        'options' => $inv_pay_list,
                        'desc' => __( 'Select KashFlow Payment Method.', 'ds-kashflow' ),
                        'id'         => 'ds_kashflow_gw_' . $k
                );
            }
            $gws[] = array( 'type' => 'sectionend', 'id' => 'ds_wcgateways_settings' ); //section end
            $woocommerce_settings[DS_KASHFLOW_ACCOUNTS] = array_merge_recursive($woocommerce_settings[DS_KASHFLOW_ACCOUNTS], $gws);
            
            $api = array();            
            $api[] = array( 'title' => __( 'Debugging', 'ds-kashflow' ), 'type' => 'title', 'desc' => '', 'id' => 'ds_kashflow_debugging' ); //section start
            $api[] = array(
                    'title' => __( 'Test API', 'ds-kashflow' ),
                    'type'         => 'dstestapi',
                    'label' => __('Test', 'ds-kashflow' ),
                    'desc' => __('unknown', 'ds-kashflow' ),
                    'id'         => 'ds_kashflow_test_api'
                );
            $api[] = array( 'type' => 'sectionend', 'id' => 'ds_kashflow_debugging' ); //section end
            $woocommerce_settings[DS_KASHFLOW_ACCOUNTS] = array_merge_recursive($woocommerce_settings[DS_KASHFLOW_ACCOUNTS], $api);
        
        // setup test api
        $nonce = wp_create_nonce( 'test_api' );
        ?>
        <script  type='text/javascript'>
        <!--
        jQuery(window).load(function(){
            jQuery("#ds_kashflow_test_api").click(function(){
                jQuery.ajax({
                    type: "post",url: "admin-ajax.php",data: { action: 'test_api', _ajax_nonce: '<?php echo $nonce; ?>' },
                    success: function(html){
                        jQuery("#testApiResult").html(html);
                    }
                });
            });
        });
        -->
        </script>
        <?php
    }
    /**
    * admin_field_label
    * 
    * custom admin field  - label
    * 
    * @param array $value
    */
    public function admin_field_label($value)
    {
        ?>
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html( $value['title'] ); ?></label>
                <?php echo $tip; ?>
            </th>
            <td class="forminp forminp-<?php echo sanitize_title( $value['type'] ) ?>">
                <label
                    name="<?php echo esc_attr( $value['id'] ); ?>"
                    id="<?php echo esc_attr( $value['id'] ); ?>"
                    style="<?php echo esc_attr( $value['css'] ); ?>"
                    class="<?php echo esc_attr( $value['class'] ); ?>"
                    ><?php echo esc_attr( $value['label']); ?></label><?php echo $description; ?>
            </td>
        </tr>
        <?php

    }
    /**
    * admin_field_testapi
    * 
    * custom admin field  - testapi (button & description)
    * 
    * @param array $value
    */
    public function admin_field_testapi($value)
    {
        global $woocommerce;
        $payment_methods = $this->api->get_inv_pay_methods();
        $gateways = $woocommerce->payment_gateways();
        
        $description = '<span id="testApiResult" class="description">' . $value['desc'] . '</span>';
        ?>
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html( $value['title'] ); ?></label>
                <?php echo $tip; ?>
            </th>
            <td class="forminp forminp-<?php echo sanitize_title( $value['type'] ) ?>">
                <input
                    name="<?php echo esc_attr( $value['id'] ); ?>"
                    id="<?php echo esc_attr( $value['id'] ); ?>"
                    type="button"
                    style="<?php echo esc_attr( $value['css'] ); ?>"
                    value="<?php echo esc_attr( $value['label']); ?>"
                    class="<?php echo esc_attr( $value['class'] ); ?>"
                    /> <?php echo $description; ?>
            </td>
        </tr>
        <?php
    } 
    
    /**
    * ajax callback for Test API
    * 
    */
    public function test_api_callback()
    {
        check_ajax_referer('test_api');
        $res = $this->api->get_company_details();
        
        if(!$res)
        {
            echo __('Failed to connect to API: ', 'ds-kashflow') . $this->api->get_last_error();
        }
        else
        {
            echo __('Connected to API successfully', 'ds-kashflow');
        }
        die();
    }     
    
    /**
    * admin_settings
    * 
    * displays the admin fields for KashFlow accounts
    *      
    */
    public function admin_settings()
    {
        global $woocommerce_settings;
        woocommerce_admin_fields( $woocommerce_settings[ DS_KASHFLOW_ACCOUNTS ] );
    }
    
    /**
    * save_admin_settings
    * 
    * saves admin settings values in db
    * 
    */
    public function save_admin_settings()
    {
        global $woocommerce_settings;
        woocommerce_update_options( $woocommerce_settings[ DS_KASHFLOW_ACCOUNTS ] );
    }
    
} // end class


} // end exist

global $ds_kashflow;
// check woocommerce is an active plugin before initializing
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) 
{
    $ds_kashflow = new Ds_Kashflow();
    
    // localization
    load_plugin_textdomain( 'ds-kashflow', false, DS_KASHFLOW_PLUGINPATH . '/languages' );  
}

?>